@@ -249,15 +249,18 @@ log_csv_parser_process(LogParser *s, LogMessage *msg, const gchar *input)
           else
             log_msg_set_value(msg, log_msg_get_value_handle((gchar *) cur_column->data), src, len);
             
-          if (*delim == 0)
-            break;
-          src = (gchar *) delim + 1;
+          src = (gchar *) delim;
+          if (*src)
+            src++;
           cur_column = cur_column->next;
 
           if (cur_column && cur_column->next == NULL && self->flags & LOG_CSV_PARSER_GREEDY)
             {
+              /* greedy mode, the last column gets it all, without taking escaping, quotes or anything into account */
               log_msg_set_value(msg, log_msg_get_value_handle((gchar *) cur_column->data), src, -1);
               cur_column = NULL;
+              src = NULL;
+              break;
             }
         }
     }
@@ -369,15 +372,22 @@ log_csv_parser_process(LogParser *s, LogMessage *msg, const gchar *input)
 
               if (cur_column && cur_column->next == NULL && self->flags & LOG_CSV_PARSER_GREEDY)
                 {
+                  /* greedy mode, the last column gets it all, without taking escaping, quotes or anything into account */
                   log_msg_set_value(msg, log_msg_get_value_handle((gchar *) cur_column->data), src, -1);
                   cur_column = NULL;
+                  src = NULL;
+                  break;
                 }
             }
         }
       g_string_free(current_value, TRUE);
     }
-  if (!cur_column && (self->flags & LOG_CSV_PARSER_DROP_INVALID))
-    return FALSE;
+  if ((cur_column || (src && *src)) && (self->flags & LOG_CSV_PARSER_DROP_INVALID))
+    {
+      /* there are unfilled variables, OR not all of the input was processed
+       * and "drop-invalid" flag is specified */
+      return FALSE;
+    }
   return TRUE;
 }
 
@@ -446,6 +456,8 @@ static void
 log_db_parser_reload_database(LogDBParser *self)
 {
   struct stat st;
+  LogPatternDatabase db_old;
+  gchar *db_file_old;
 
   if (stat(self->db_file, &st) < 0)
     {
@@ -460,11 +472,15 @@ log_db_parser_reload_database(LogDBParser *self)
     }
   self->db_file_inode = st.st_ino;
   self->db_file_mtime = st.st_mtime;
+  db_old = self->db;
+  db_file_old = self->db_file;
 
-  log_pattern_database_free(&self->db);
   if (!log_pattern_database_load(&self->db, self->db_file))
     {
-      msg_error("Error reloading pattern database, no pattern recognition will be done", NULL);
+      msg_error("Error reloading pattern database, no automatic reload will be performed", NULL);
+      /* restore the old object pointers */
+      self->db = db_old;
+      self->db_file = db_file_old;
     }
   else
     {
@@ -473,6 +489,8 @@ log_db_parser_reload_database(LogDBParser *self)
                  evt_tag_str("version", self->db.version),
                  evt_tag_str("pub_date", self->db.pub_date),
                  NULL);
+      /* free the old database if the new was loaded successfully */
+      log_pattern_database_free(&db_old);
     }
 
 }
