@@ -311,8 +311,8 @@ log_reader_fetch_log(LogReader *self, LogProto *proto)
     parse_flags |= LP_VALIDATE_UTF8;
   if (self->options->options & LRO_NO_MULTI_LINE)
     parse_flags |= LP_NO_MULTI_LINE;
-  if (self->options->options & LRO_STORE_LEGACY_MSGHDR)
-    parse_flags |= LP_STORE_LEGACY_MSGHDR;
+  if (self->options->options & LRO_DONT_STORE_LEGACY_MSGHDR)
+    parse_flags |= LP_DONT_STORE_LEGACY_MSGHDR;
     
   if (self->waiting_for_preemption)
     may_read = FALSE;
@@ -825,7 +825,9 @@ log_reader_options_lookup_flag(const gchar *flag)
   if (strcmp(flag, "no-multi-line") == 0 || strcmp(flag, "no_multi_line") == 0)
     return LRO_NO_MULTI_LINE;
   if (strcmp(flag, "store-legacy-msghdr") == 0 || strcmp(flag, "store_legacy_msghdr") == 0)
-    return LRO_STORE_LEGACY_MSGHDR;
+    return 0;
+  if (strcmp(flag, "dont-store-legacy-msghdr") == 0 || strcmp(flag, "dont_store_legacy_msghdr") == 0)
+    return LRO_DONT_STORE_LEGACY_MSGHDR;
   if (strcmp(flag, "empty-lines") == 0 || strcmp(flag, "empty_lines") == 0)
     return LRO_EMPTY_LINES;
   msg_error("Unknown parse flag", evt_tag_str("flag", flag), NULL);
