@@ -159,9 +159,18 @@ log_writer_fd_prepare(GSource *source,
         }
       else
         {
-          *timeout = g_time_val_diff(&self->flush_target, &now) / 1000;
-          if (*timeout < 0)
-            return TRUE;
+          glong to = g_time_val_diff(&self->flush_target, &now) / 1000;
+          if (to <= 0)
+            {
+              /* timeout elapsed, start polling again */
+              if (self->writer->flags & LW_ALWAYS_WRITABLE)
+                return TRUE;
+              self->pollfd.events = proto_cond;
+            }
+          else
+            {
+              *timeout = to;
+            }
         }
       return FALSE;
     }
@@ -213,9 +222,13 @@ log_writer_fd_check(GSource *source)
       if (self->flush_waiting_for_timeout)
         {
           GTimeVal tv;
+
           /* check if timeout elapsed */
           g_source_get_current_time(source, &tv);
-          return self->flush_target.tv_sec <= tv.tv_sec || (self->flush_target.tv_sec == tv.tv_sec && self->flush_target.tv_usec <= tv.tv_usec);
+          if (!(self->flush_target.tv_sec <= tv.tv_sec || (self->flush_target.tv_sec == tv.tv_sec && self->flush_target.tv_usec <= tv.tv_usec)))
+            return FALSE;
+          if ((self->writer->flags & LW_ALWAYS_WRITABLE))
+            return TRUE;
         }
     }
   return !!(self->pollfd.revents & (G_IO_OUT | G_IO_ERR | G_IO_HUP | G_IO_IN));
@@ -350,7 +363,15 @@ log_writer_last_msg_flush(LogWriter *self)
   log_msg_set_value(m, LM_V_MESSAGE, buf, len);
 
   path_options.flow_control = FALSE;
-  log_queue_push_tail(self->queue, m, &path_options);
+  if (!log_queue_push_tail(self->queue, m, &path_options))
+    {
+      stats_counter_inc(self->dropped_messages);
+      msg_debug("Destination queue full, dropping suppressed message",
+                evt_tag_int("queue_len", log_queue_get_length(self->queue)),
+                evt_tag_int("mem_fifo_size", self->options->mem_fifo_size),
+                NULL);
+      log_msg_drop(m, &path_options);
+    }
 
   log_writer_last_msg_release(self);
 }
@@ -729,6 +750,9 @@ log_writer_flush_log(LogWriter *self, LogProto *proto)
         {
           /* push back to the queue */
           log_queue_push_head(self->queue, lm, &path_options);
+
+          /* force exit of the loop */
+          num_elements = 1;
         }
         
       msg_set_context(NULL);
